% Friday 4, December 2015
% Copy-Paste and modifications of the script "script01_Algo1.m" in
% ~/thesis/Work_In_Progress/sorted_151203_MyBFCorrection
% Used in a Demo to compare the different ways to do a regression in the
% Bilateral Filter while keeping reasonnable times of execution.
% 

function BFreg_fast_newpond(img_in_name,radius,sigma_s,sigma_r,...
                            img_out_name,img_diff_name,diff_coeffmultp)

% Warning: please give "sigma_r" with respect to 100.

% /// READING PARAMETERS ///

sigma_r         = str2double(sigma_r)/100;
sigma_s         = str2double(sigma_s);
radius          = str2double(radius);
diff_coeffmultp = str2double(diff_coeffmultp);

% /// READING INPUT IMAGE ///

input = imread(img_in_name);
if     isa(input,'uint8' ), img_in = double(input)/255;
elseif isa(input,'uint16'), img_in = double(input)/65535;
else error(...
 'BFreg_fast_regbygrad: error while loading image (not uint8 nor uint16)');
end
if size(input,3)>1, img_in = sum(img_in,3)/size(input,3);
end

% /// PRE-PROCESSING ///

% Memory allocation
[R,C]  = size(img_in);
bary_r = zeros(R,C);
bary_c = zeros(R,C);
bary_r2= zeros(R,C);
bary_c2= zeros(R,C);
img_out = zeros(R,C);
% bary_rc= zeros(R,C);

% Space weighting
[mapC,mapR] = meshgrid(-radius:radius,-radius:radius);
gaussS      = exp(-(mapC.^2 + mapR.^2)/(2*sigma_s^2));

% Image edges management : mirror copy
img_gr_mirr = mirror(img_in,radius);

% /// FILTERING ///

for r = 1+radius:R+radius
    for c = 1+radius:C+radius
        
        % Space AND Range original weights
        mapW    = img_gr_mirr(r-radius:r+radius,c-radius:c+radius) - img_gr_mirr(r,c);
        gaussR  = exp(-mapW.^2/(2*sigma_r^2));
        W       = gaussS.*gaussR;
        W       = W./sum(W(:));
        
        % Barycenter (and more)
        bary_r(r-radius,c-radius) = sum(W(:).*mapR(:));
        bary_c(r-radius,c-radius) = sum(W(:).*mapC(:));
        bary_r2(r-radius,c-radius) = sum(W(:).*mapR(:).^2);
        bary_c2(r-radius,c-radius) = sum(W(:).*mapC(:).^2);
        
        % /// SUPPLEMENTARY ///
        % bary_rc(r-radius,c-radius) = sum(W(:).*mapR(:).*mapC(:));
        % ///
        
        % Weights modification
        WG = W.*(1 - bary_r(r-radius,c-radius).*(mapR/bary_r2(r-radius,c-radius)) ...
                   - bary_c(r-radius,c-radius).*(mapC/bary_c2(r-radius,c-radius)));
        WG = WG./sum(WG(:));
        
        % Computation of the output image
        img_out(r-radius,c-radius) = sum(sum(WG.*img_gr_mirr(r-radius:r+radius,c-radius:c+radius)));
         
    end
end

% /// WRITTING IMAGES ///
% imwrite(uint16(65535*img_out),img_out_name);
% imwrite(uint16(65535*(.5 + diff_coeffmultp.*(img_in - img_out))),img_diff_name);
imwrite(uint8(255*img_out),img_out_name);
imwrite(uint8(255*(.5 + diff_coeffmultp.*(img_in - img_out))),img_diff_name);

end


