% Friday 4, december 2015
% Function for the demo on "Fast Bilateral Filters with Regression"
% Copy-Past, plus modification, of the script "script03_NewRegByGRAD_true"
% in the directory
% ~/thesis/Work_In_Progress/sorted_151120_BF_NewRegression/
% 
% The modifications aim to transform the script in function, simplified as
% possible what is done, removing the "test" parts, etc.

% /// SPECIFICATIONS ///
% Take a color OR gray image as input. Colored image are converted to gray.
% Give a gray filtered image as output
% ///

function BFreg_fast_regbygrad(img_in_name,radius,sigma_s,sigma_r,...
                              grad_blendparam,img_out_name,img_diff_name,...
                              diff_coeffmultp)
% Warning: please give "sigma_r" with respect to 100.
% Warning: idem for "grad_blendparam"
                            
% /// READING PARAMETERS ///

sigma_r         = str2double(sigma_r)/100;
sigma_s         = str2double(sigma_s);
radius          = str2double(radius);
grad_blendparam = str2double(grad_blendparam)/100;
diff_coeffmultp = str2double(diff_coeffmultp);

% /// READING INPUT IMAGE ///

input = imread(img_in_name);
if     isa(input,'uint8' ), img_in = double(input)/255;
elseif isa(input,'uint16'), img_in = double(input)/65535;
else error(...
 'BFreg_fast_regbygrad: error while loading image (not uint8 nor uint16)');
end
if size(input,3)>1, img_in = sum(img_in,3)/size(input,3);
end

% /// GRADIENT COMPUTATION ///

% definitions...
grad_K_left  = [+1 -1  0];
grad_K_right = [ 0 +1 -1];
pond         = @(g1,g2,s)( max(0, .5 - ( abs(g1 - g2)/s )) );

% Gradient for axis R (rows)
grad_r_left  = conv2(mirror(img_in,[1 0]),grad_K_left' ,'valid');
grad_r_right = conv2(mirror(img_in,[1 0]),grad_K_right','valid');
grad_r_all   = cat(3,grad_r_left,grad_r_right);
[~,pos_r]    = min( abs(grad_r_all) , [], 3 );

grad_r  = ( 1 - pond(grad_r_left,grad_r_right,grad_blendparam) ).* ...
          (grad_r_left.*(pos_r == 1) + grad_r_right.*(pos_r == 2)) + ...
          (     pond(grad_r_left,grad_r_right,grad_blendparam) ).* ...
          (grad_r_left.*(pos_r == 2) + grad_r_right.*(pos_r == 1)) ;

% Gradient for axis C (columns)
grad_c_left  = conv2(mirror(img_in,[0 1]),grad_K_left  ,'valid');
grad_c_right = conv2(mirror(img_in,[0 1]),grad_K_right ,'valid');
grad_c_all   = cat(3,grad_c_left,grad_c_right);
[~,pos_c]    = min( abs(grad_c_all) , [], 3 );

grad_c  = ( 1 - pond(grad_c_left,grad_c_right,grad_blendparam) ).* ...
          (grad_c_left.*(pos_c == 1) + grad_c_right.*(pos_c == 2)) + ...
          (     pond(grad_c_left,grad_c_right,grad_blendparam) ).* ...
          (grad_c_left.*(pos_c == 2) + grad_c_right.*(pos_c == 1)) ;

% /// PRE-PROCESSING ///

% Memory allocation
[R,C]  = size(img_in);
bary_r = zeros(R,C);
bary_c = zeros(R,C);
coef_a = zeros(R,C);
coef_b = zeros(R,C);
coef_c = zeros(R,C);
img_out = zeros(R,C);

% Space weighting
[mapC,mapR] = meshgrid(-radius:radius,-radius:radius);
gaussS      = exp(-(mapC.^2 + mapR.^2)/(2*sigma_s^2));

% Image edges management : mirror copy
img_gr_mirr = mirror(img_in,radius);
grad_h_mirr = mirror(grad_r,radius);
grad_w_mirr = mirror(grad_c,radius);

% /// FILTERING ///

for r = 1+radius:R+radius
    for c = 1+radius:C+radius
        
        % Space AND Range weights
        mapW    = img_gr_mirr(r-radius:r+radius,c-radius:c+radius) - img_gr_mirr(r,c);
        gaussR  = exp(-mapW.^2/(2*sigma_r^2));
        W       = gaussS.*gaussR;

        % Weights normalization
        W       = W./sum(W(:));
        
        % Barycenter
        bary_r(r-radius,c-radius) = sum(W(:).*mapR(:));
        bary_c(r-radius,c-radius) = sum(W(:).*mapC(:));

        % Estimation of the regression coefficients
        coef_a(r-radius,c-radius) = sum(sum(W.*grad_h_mirr(r-radius:r+radius,c-radius:c+radius)));
        coef_b(r-radius,c-radius) = sum(sum(W.*grad_w_mirr(r-radius:r+radius,c-radius:c+radius)));
        coef_c(r-radius,c-radius) = sum(sum(W.*img_gr_mirr(r-radius:r+radius,c-radius:c+radius)));

        % Computation of the output image
        img_out(r-radius,c-radius) = ...
            - coef_a(r-radius,c-radius).*bary_r(r-radius,c-radius) ...
            - coef_b(r-radius,c-radius).*bary_c(r-radius,c-radius) ...
            + coef_c(r-radius,c-radius);
    end
end

% /// WRITTING IMAGES ///
% imwrite(uint16(65535*img_out),img_out_name);
% imwrite(uint16(65535*(.5 + diff_coeffmultp.*(img_in - img_out))),img_diff_name);
imwrite(uint8(255*img_out),img_out_name);
imwrite(uint8(255*(.5 + diff_coeffmultp.*(img_in - img_out))),img_diff_name);

end