#include <cstdio>
#include <cstdlib>
#include <cstring>
#include "SparseFilter.h"
#include "IioImage.h"

using sparse_filter::IioImage;
using sparse_filter::SparseFilter;

// c: pointer to original argc
// v: pointer to original argv
// o: option name after hyphen
// d: default value (if NULL, the option takes no argument)
const char *pick_option(int *c, char **v, const char *o, const char *d) {
    int id = d ? 1 : 0;
    for (int i = 0; i < *c - id; i++) {
        if (v[i][0] == '-' && 0 == strcmp(v[i] + 1, o)) {
            char *r = v[i + id] + 1 - id;
            for (int j = i; j < *c - id; j++)
                v[j] = v[j + id + 1];
            *c -= id + 1;
            return r;
        }
    }
    return d;
}

int main(int argc, char *argv[]) {
    if (argc < 6) {
        fprintf(stderr, "too few parameters\n");
        fprintf(stderr, "usage: %s input sigma_d sigma_r output difference [-d d] [-r r] [-t threshold] [-m 1 for reg]\n", argv[0]);
        fprintf(stderr, "Default parameters: d = 3 ; r = 3*sigma_d ; t = 1 ; m = -1\n");
        return 1;
    }
    IioImage source(argv[1]);
    IioImage output;
    IioImage diff;
    float coeffmult = atof(pick_option(&argc, argv, "d", "3"));
    float sigma_d = atof(argv[2]);
    float sigma_r = atof(argv[3]);
    int r = atoi(pick_option(&argc, argv, "r", "-1"));
    if (r == -1) r = int(3 * sigma_d);
    float threshold = atof(pick_option(&argc, argv, "t", "1"));
    
    int reg = atoi(pick_option(&argc, argv, "m", "-1"));
    
    // Trois fonctions sont possibles :
    // 1) Sparse-Filter (fonction de Nicola)
    // 2) BilateralFilter : filtre bilatéral standard
    // 3) BilateralFilterReg : filter bilatéral avec regression
    
    /*
    SparseFilter(      source, sigma_d * sigma_d, sigma_r * sigma_r, r, threshold, &output);
     */
    if(reg == 1) {
        printf("Application of the Bilateral Filter with Regression.\n");
        BilateralFilterReg(source, sigma_d * sigma_d, sigma_r * sigma_r, r, &output);
    }
    else {
        printf("Application of the Standard Bilateral Filter.\n");
        BilateralFilter(   source, sigma_d * sigma_d, sigma_r * sigma_r, r, &output);
    }
    
    difference(source, output, &diff, coeffmult);
    
    // Sauvegarde de l'image de sortie
    output.Save(argv[4]);
    diff.Save(argv[5]);

    return 0;
}
