function img_out = mirror(img_in, pad)
% the "mirror" function replace the "miroir" function. The new one is able
% to pad the input image with more pixels than its own size for each side.
% i.e. the vector "pad" can be bigger than [H,W] with H and W the Height
% and Width of the input image.
%
% Usage :
%   img_padded = mirror(img,[ height_pad, width_pad ]);
%   img_padded = mirror(img,pad);
%
% In the second case, height_pad = pad, width_pad = pad;
%
%

if size(img_in,3) > 1
    error('mirror: the input image must be 1 channel only.');
end

if isscalar(pad), pad = [pad,pad]; end
% if size(pad) == [2,1], pad = [pad(1),pad(2)]; end

[H,W] = size(img_in);
img_out = img_in;

% Up and Down
while(pad(1) >= H)
    img_out = [ flipud(img_out(1:H,:)) ;...
                img_out ;...
                flipud(img_out(end+1-H:end,:)) ];
    pad     = pad - [H,0];
end
if pad(1) ~= 0
    img_out = [ flipud(img_out(1:pad(1),:)); ...
                img_out; ...
                flipud(img_out(end+1-pad(1):end,:)) ];
end

% Left and Right
while(pad(2) >= W)
    img_out = [ fliplr(img_out(:,1:W)) ,...
                img_out ,...
                fliplr(img_out(:,end+1-W:end)) ];
    pad     = pad - [0,W];
end
if pad(2) ~= 0
    img_out = [ fliplr(img_out(:,1:pad(2))), ...
                img_out, ...
                fliplr(img_out(:,end+1-pad(2):end))];
end
end
